@@ -517,7 +517,7 @@ def get_download_counts():
     return counts
 
 
-def get_install_deps(module_name, version=None, skip_installed=True):
+def get_install_deps(module_name, version=None, skip_installed=True, recursive_depth=0):
     mic.update_remote()
     # If input module version not provided, set to highest
     if version is None:
@@ -547,6 +547,15 @@ def get_install_deps(module_name, version=None, skip_installed=True):
         # Dont include if no matching version exists
         if highest_matching is not None:
             deps[req.name] = highest_matching
+            # Recursively get dependencies
+            if (recursive_depth < 3):
+                deps = dict(get_install_deps(
+                                module_name=req.name, 
+                                version=highest_matching,
+                                skip_installed=skip_installed,
+                                recursive_depth=recursive_depth + 1
+                            ), **deps)
+
     return deps
 
 
@@ -848,6 +857,8 @@ def get_system_conf(file_only=False):
             conf[constants.jobs_dir_key] = constants.default_jobs_dir
         if constants.log_dir_key not in conf:
             conf[constants.log_dir_key] = constants.default_log_dir
+        if constants.metrics_dir_key not in conf:
+            conf[constants.metrics_dir_key] = constants.default_metrics_dir
         key = "num_input_line_warning_cutoff"
         if key not in conf:
             conf[key] = constants.default_num_input_line_warning_cutoff
@@ -860,6 +871,10 @@ def get_system_conf(file_only=False):
         key = "max_num_concurrent_annotators_per_job"
         if key not in conf:
             conf[key] = constants.default_max_num_concurrent_annotators_per_job
+        if constants.save_metrics_key not in conf:
+            conf[constants.save_metrics_key] = constants.default_save_metrics
+        if constants.metrics_url_key not in conf:
+            conf[constants.metrics_url_key] = constants.default_metrics_url
         if "custom_system_conf" in globals():
             global custom_system_conf
             for k, v in custom_system_conf.items():
@@ -1559,6 +1574,24 @@ def set_modules_dir(path, overwrite=False):
             overwrite_conf_path = get_main_default_path()
         shutil.copy(overwrite_conf_path, get_main_conf_path())
 
+def set_metrics_config(value, overwrite=False):
+    """
+    Set the save_metrics configuration in the system conf file.
+    """
+    update_system_conf_file({constants.save_metrics_key: value})
+
+def get_metrics_config():
+    """
+    Get the current modules directory
+    """
+    conf = get_system_conf()
+    metrics_conf = conf[constants.save_metrics_key]
+    if metrics_conf == True:
+        return 'Y'
+    elif metrics_conf == False:
+        return 'N'
+    else:
+        return metrics_conf
 
 # return a list of module types (e.g. annotators) in the local install
 def show_cravat_conf():
@@ -1635,3 +1668,27 @@ def update_mic():
 
 
 mic = ModuleInfoCache()
+
+#Used to create an oc package from a user job.  The details (like annotators run) from that job are 
+#provided in a dictionary.  A package is created and then can be used for future jobs with the --package option
+def create_package(name, pkg_details, overwrite):
+    pkg_dir = os.path.join(get_modules_dir(), "packages")
+    if not (os.path.exists(pkg_dir)):    
+        os.mkdir(pkg_dir)
+    pkg_path = os.path.join(pkg_dir, name)
+    
+    #if package already exists, error out
+    if os.path.exists(pkg_path):
+        if overwrite == False:
+            raise ValueError('Duplicate package already exists')
+    else:
+        os.mkdir(pkg_path)
+        
+    #create the package directory, an empty package python file and the package yml file
+    if not (os.path.join(pkg_dir, name)): 
+        os.mkdir(os.path.join(pkg_dir, name))
+    open(os.path.join(pkg_path, name + ".py"), 'a').close()
+    with open(os.path.join(pkg_path, name + ".yml"), "w") as wf:
+        wf.write(yaml.dump(pkg_details, default_flow_style=False))
+    mic.update_local()
+
