@@ -2,12 +2,13 @@ import time
 import argparse
 import os
 import sys
+from cravat import cravat_metrics as metrics
 from cravat import admin_util as au
 from cravat import util
 from cravat.config_loader import ConfigLoader
 from cravat.util import write_log_msg
 import aiosqlite
-import datetime
+from datetime import datetime, timezone
 from types import SimpleNamespace
 from cravat import constants
 import json
@@ -28,6 +29,9 @@ from cravat.inout import CravatWriter
 from cravat.inout import CravatReader
 import glob
 import nest_asyncio
+from queue import Empty
+    
+import sys
 
 nest_asyncio.apply()
 import re
@@ -193,6 +197,14 @@ cravat_cmd_parser.add_argument(
     default=None,
     help="Leave temporary files after run is complete.",
 )
+# For GUI use only. Not shown in help text
+cravat_cmd_parser.add_argument(
+    "--keep-status",
+    dest="keep_status",
+    action="store_true",
+    default=None,
+    help=argparse.SUPPRESS,
+)
 cravat_cmd_parser.add_argument(
     "--writeadmindb",
     dest="writeadmindb",
@@ -321,6 +333,7 @@ class Cravat(object):
         self.append_mode = False
         self.pipeinput = False
         try:
+            self.metricObj = metrics.cravatMetrics()
             self.make_args_namespace(kwargs)
             if self.args.clean_run:
                 if not self.args.silent:
@@ -375,7 +388,7 @@ class Cravat(object):
                 self.status_json["orig_input_path"] = self.inputs
                 self.status_json[
                     "submission_time"
-                ] = datetime.datetime.now().isoformat()
+                ] = datetime.now().isoformat()
                 self.status_json["viewable"] = False
                 self.status_json["note"] = self.args.note
                 self.status_json["status"] = "Starting"
@@ -404,7 +417,7 @@ class Cravat(object):
                 os.path.basename(x) for x in self.inputs
             ]
             self.status_json["orig_input_path"] = self.inputs
-            self.status_json["submission_time"] = datetime.datetime.now().isoformat()
+            self.status_json["submission_time"] = datetime.now().isoformat()
             self.status_json["viewable"] = False
             self.status_json["note"] = self.args.note
             self.status_json["status"] = "Starting"
@@ -490,6 +503,9 @@ class Cravat(object):
         no_problem_in_run = True
         report_response = None
         try:
+            if au.get_metrics_config() == "empty":
+                print("NOTICE: OpenCRAVAT gathers metrics to report usage to funders, and improve the tool. No private data is collected. For more details, visit our documentation at: https://open-cravat.readthedocs.io/en/latest/Metrics.html.")
+                au.update_system_conf_file({"save_metrics": True})
             self.aggregator_ran = False
             self.update_status("Started cravat", force=True)
             if self.pipeinput == False:
@@ -499,6 +515,7 @@ class Cravat(object):
             if not self.args.silent:
                 print("Input file(s): {}".format(input_files_str))
                 print("Genome assembly: {}".format(self.input_assembly))
+            self.metricObj.set_job_data('genome',self.input_assembly)
             self.logger.info("input files: {}".format(input_files_str))
             self.logger.info("input assembly: {}".format(self.input_assembly))
             self.log_versions()
@@ -516,6 +533,7 @@ class Cravat(object):
                 rtime = time.time() - stime
                 if not self.args.silent:
                     print("finished in {0:.3f}s".format(rtime))
+                self.metricObj.set_job_converter('runtime',round(rtime,3))
                 converter_ran = True
                 if self.numinput == 0:
                     msg = "No variant found in input"
@@ -540,6 +558,7 @@ class Cravat(object):
                 else:
                     self.run_genemapper()
                 rtime = time.time() - stime
+                self.metricObj.set_job_mapper('runtime',round(rtime,3))
                 if not self.args.silent:
                     print("finished in {0:.3f}s".format(rtime))
                 self.mapper_ran = True
@@ -605,9 +624,22 @@ class Cravat(object):
             self.handle_exception(e)
             no_problem_in_run = False
         finally:
+            dbpath = os.path.join(self.output_dir, self.run_name + ".sqlite")
+
+            # Set journal_mode
+            dbconn = sqlite3.connect(dbpath)
+            try:
+                dbconn.execute('pragma journal_mode="delete"')
+            except sqlite3.Error as e:
+                pass
+            finally:
+                dbconn.close()
+
             end_time = time.time()
             display_time = time.asctime(time.localtime(end_time))
             runtime = end_time - self.start_time
+            self.metricObj.set_job_data('jobRuntime',round(runtime,3))
+            success = "Finished Normally"
             if no_problem_in_run:
                 self.logger.info("finished: {0}".format(display_time))
                 self.logger.info("runtime: {0:0.3f}s".format(runtime))
@@ -616,12 +648,15 @@ class Cravat(object):
             else:
                 self.logger.info("finished with an exception: {0}".format(display_time))
                 self.logger.info("runtime: {0:0.3f}s".format(runtime))
+                success = "Finished with an exception"
                 if not self.args.silent:
                     print(
                         "Finished with an exception. Runtime: {0:0.3f}s".format(runtime)
                     )
                     print("Check {}".format(self.log_path))
                 self.update_status("Error", force=True)
+            self.metricObj.set_job_data('success',success)
+            self.metricObj.do_job_metrics(self)
             self.close_logger()
             if self.args.do_not_change_status != True:
                 self.status_writer.flush()
@@ -658,6 +693,7 @@ class Cravat(object):
         await db.close()
 
     def write_smartfilters(self):
+        ttime = time.time()
         if not self.args.silent:
             print("Indexing")
         dbpath = os.path.join(self.output_dir, self.run_name + ".sqlite")
@@ -707,13 +743,23 @@ class Cravat(object):
                     if not self.args.silent:
                         print(f"\tfinished in {time.time()-st:.3f}s")
 
-        # Package filter
+            rtime = time.time() - ttime
+            self.metricObj.set_job_data('indexingRuntime',round(rtime,3))
+        # Package filter and viewer settings
         if hasattr(self.args, "filter") and self.args.filter is not None:
             q = "create table if not exists viewersetup (datatype text, name text, viewersetup text, unique (datatype, name))"
             cursor.execute(q)
-            filter_set = json.dumps({"filterSet": self.args.filter})
+            #filter_set = json.dumps({"filterSet": self.args.filter})
+            filter_set = self.args.filter
             q = f'insert or replace into viewersetup values ("filter", "quicksave-name-internal-use", \'{filter_set}\')'
             cursor.execute(q)
+        if hasattr(self.args, "viewer") and self.args.viewer is not None:
+            q = "create table if not exists viewersetup (datatype text, name text, viewersetup text, unique (datatype, name))"
+            cursor.execute(q)
+            viewer_setting = self.args.viewer
+            q = f'insert or replace into viewersetup values ("layout", "quicksave-name-internal-use", \'{viewer_setting}\')'
+            cursor.execute(q)
+
         conn.commit()
         cursor.close()
         conn.close()
@@ -850,9 +896,6 @@ class Cravat(object):
 
     def download_url_input(self, input_no):
         ip = self.inputs[input_no]
-        if " " in ip:
-            print(f"Space is not allowed in input file paths ({ip})")
-            exit()
         if util.is_url(ip):
             import requests
             if not self.args.silent:
@@ -1412,6 +1455,7 @@ class Cravat(object):
         # Variant level
         if not self.args.silent:
             print("\t{0:30s}\t".format("Variants"), end="", flush=True)
+        ttime = time.time()
         stime = time.time()
         cmd = [
             "donotremove",
@@ -1440,7 +1484,8 @@ class Cravat(object):
         rtime = time.time() - stime
         if not self.args.silent:
             print("finished in {0:.3f}s".format(rtime))
-
+        variants = {'runtime': round(rtime,3)}
+        self.metricObj.set_job_aggregator('variants',variants)
         # Gene level
         if not self.args.silent:
             print("\t{0:30s}\t".format("Genes"), end="", flush=True)
@@ -1470,6 +1515,8 @@ class Cravat(object):
         rtime = time.time() - stime
         if not self.args.silent:
             print("finished in {0:.3f}s".format(rtime))
+        genes = {'runtime': round(rtime,3)}
+        self.metricObj.set_job_aggregator('genes',genes)
 
         # Sample level
         if not self.append_mode:
@@ -1500,6 +1547,8 @@ class Cravat(object):
             rtime = time.time() - stime
             if not self.args.silent:
                 print("finished in {0:.3f}s".format(rtime))
+            samples = {'runtime': round(rtime,3)}
+            self.metricObj.set_job_aggregator('samples',samples)
 
         # Mapping level
         if not self.append_mode:
@@ -1529,7 +1578,12 @@ class Cravat(object):
             rtime = time.time() - stime
             if not self.args.silent:
                 print("finished in {0:.3f}s".format(rtime))
+            tags = {'runtime': round(rtime,3)}
+            self.metricObj.set_job_aggregator('tags',tags)
 
+        rtime = time.time() - ttime
+        total = {'runtime': round(rtime,3)}
+        self.metricObj.set_job_aggregator('total',total)
         return v_aggregator.db_path
 
     def run_postaggregators(self):
@@ -1557,6 +1611,11 @@ class Cravat(object):
             post_agg.run()
             rtime = time.time() - stime
             if not self.args.silent and post_agg.should_run_annotate:
+                postagg = {}
+                postagg['name'] = module.name
+                postagg['version'] = module.conf['version']
+                postagg['runtime'] = round(rtime,3)
+                self.metricObj.set_job_post_aggregator(postagg)
                 print("finished in {0:.3f}s".format(rtime))
 
     async def run_reporter(self):
@@ -1610,7 +1669,10 @@ class Cravat(object):
                 arg_dict["filtersql"] = self.args.filtersql
                 arg_dict['includesample'] = self.args.includesample
                 arg_dict['excludesample'] = self.args.excludesample
-                arg_dict["filter"] = self.args.filter
+                if self.args.filter is not None:
+                    arg_dict["filter"] = json.loads(self.args.filter)["filterSet"]
+                else:
+                    arg_dict["filter"] = self.args.filter
                 arg_dict["filterpath"] = self.args.filterpath
                 Reporter = util.load_class(module.script_path, "Reporter")
                 reporter = Reporter(arg_dict)
@@ -1723,6 +1785,7 @@ class Cravat(object):
         end_queue = self.manager.Queue()
         all_mnames = set(self.run_annotators)
         queued_mnames = set()
+        done_annots = []
         done_mnames = set(self.done_annotators)
         queue_populated = self.manager.Value("c_bool", False)
         pool_args = [
@@ -1745,9 +1808,9 @@ class Cravat(object):
             # TODO not handling case where parent annotator errors out
             while (queued_mnames != all_mnames):  
                 # Block until item availble in end_queue
-                finished_module = end_queue.get()
-                done_mnames.add(finished_module)
+                done_annots.append(end_queue.get(True))
                 # Queue any annotators that now have requirements complete
+                done_mnames.add(done_annots[-1]['module'])
                 for mname, module in self.run_annotators.items():
                     annotator_not_queue = mname not in queued_mnames
                     secondaries_done = set(module.secondary_module_names) <= done_mnames
@@ -1756,6 +1819,19 @@ class Cravat(object):
                         queued_mnames.add(mname)
             queue_populated = True
             pool.join()
+            # Retrieve finished annotators from end_queue
+            while True:
+                try:
+                    done_annots.append(end_queue.get(False))
+                except Empty:
+                    break
+            # Write annotator metrics
+            for done_annot in done_annots:
+                annotator = {}
+                annotator['name'] = done_annot['module']
+                annotator['version'] = done_annot['version']
+                annotator['runtime'] = done_annot['runtime']
+                self.metricObj.set_job_annotator(annotator)
         self.log_path = os.path.join(self.output_dir, self.run_name + ".log")
         self.log_handler = logging.FileHandler(self.log_path, "a")
         formatter = logging.Formatter(
@@ -1814,12 +1890,16 @@ class Cravat(object):
         dbpath = os.path.join(self.output_dir, self.run_name + ".sqlite")
         conn = await aiosqlite.connect(dbpath)
         cursor = await conn.cursor()
+        dt = datetime.now(timezone.utc)
+        utc_time = dt.replace(tzinfo=timezone.utc)    
+        utc_dt_string = utc_time.strftime("%Y/%m/%d %H:%M:%S")
         if not self.append_mode:
             q = "drop table if exists info"
             await cursor.execute(q)
             q = "create table info (colkey text primary key, colval text)"
             await cursor.execute(q)
-        modified = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
+        modified = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
+        self.metricObj.set_job_data('resultModifiedAt',utc_dt_string)
         q = (
             'insert or replace into info values ("Result modified at", "'
             + modified
@@ -1827,8 +1907,10 @@ class Cravat(object):
         )
         await cursor.execute(q)
         if not self.append_mode:
-            created = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
+            created = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
             q = 'insert into info values ("Result created at", "' + created + '")'
+            self.metricObj.set_job_data('numInputFiles',len(self.inputs))
+            self.metricObj.set_job_data('resultCreatedAt',utc_dt_string)
             await cursor.execute(q)
             q = 'insert into info values ("Input file name", "{}")'.format(
                 ";".join(self.inputs)
@@ -1843,24 +1925,29 @@ class Cravat(object):
             q = "select count(*) from variant"
             await cursor.execute(q)
             r = await cursor.fetchone()
-            no_input = str(r[0])
+            no_input = r[0]
+            self.metricObj.set_job_data('numVariants',no_input)
             q = (
                 'insert into info values ("Number of unique input variants", "'
-                + no_input
+                + str(no_input)
                 + '")'
             )
             await cursor.execute(q)
+            self.metricObj.set_job_data('ocVersion',self.pkg_ver)
             q = 'insert into info values ("open-cravat", "{}")'.format(self.pkg_ver)
             await cursor.execute(q)
-            q = 'insert into info values ("_converter_format", "{}")'.format(
-                await self.get_converter_format_from_crv()
-            )
+            converterFormat = await self.get_converter_format_from_crv()
+            q = 'insert into info values ("_converter_format", "{}")'.format(converterFormat)
             await cursor.execute(q)
+            self.metricObj.set_job_data('converterFormat',converterFormat)
+            self.metricObj.set_job_converter('name',converterFormat)
             (
                 mapper_title,
                 mapper_version,
                 mapper_modulename,
             ) = await self.get_mapper_info_from_crx()
+            self.metricObj.set_job_mapper('name',mapper_modulename)
+            self.metricObj.set_job_mapper('version',mapper_version)
             genemapper_str = "{} ({})".format(mapper_title, mapper_version)
             q = 'insert into info values ("Gene mapper", "{}")'.format(genemapper_str)
             await cursor.execute(q)
@@ -1879,6 +1966,7 @@ class Cravat(object):
                     )
                     await cursor.execute(q)
             q = f'insert into info values ("primary_transcript", "{",".join(self.args.primary_transcript)}")'
+            self.metricObj.set_job_data('primaryTranscript',"#".join(self.args.primary_transcript))
             await cursor.execute(q)
         q = 'select colval from info where colkey="annotators_desc"'
         await cursor.execute(q)
@@ -1928,8 +2016,9 @@ class Cravat(object):
         await cursor.execute(q)
         q = 'insert or replace into info values ("_annotators", "{}")'.format(
             ",".join(annotators)
-        )
+        )  
         await cursor.execute(q)
+
         await conn.commit()
         await cursor.close()
         await conn.close()
@@ -1974,7 +2063,7 @@ class Cravat(object):
                 fn_end = fn.split(".")[-1]
                 if fn_end in ["var", "gen", "crv", "crx", "crg", "crs", "crm", "crt"]:
                     os.remove(os.path.join(self.output_dir, fn))
-                if fn.split(".")[-2:] == ["status", "json"]:
+                if not self.args.keep_status and fn.split(".")[-2:] == ["status", "json"]:
                     os.remove(os.path.join(self.output_dir, fn))
 
 
