@@ -97,6 +97,12 @@ config_system_p = config_sp.add_parser(
     add_help=False,
     help="Show cravat config",
 )
+config_system_p = config_sp.add_parser(
+    "metrics",
+    parents=[cravat_admin.parser_metrics],
+    add_help=False,
+    help="set metric collection configuration",
+)
 
 # new
 new_p = root_sp.add_parser(
@@ -196,6 +202,12 @@ util_showsqliteinfo_p = util_sp.add_parser(
     add_help=False,
     help="Show SQLite result file information",
 )
+util_jobtopackage_p = util_sp.add_parser(
+    "jobtopackage",
+    parents=[cravat_util.parser_jobtopackage],
+    add_help=False,
+    help="Create a package from a previously run job",
+)
 
 # version
 version_p = root_sp.add_parser(
