@@ -102,7 +102,7 @@ class FilterManager {
 		this.vpropQbId = 'vprop-qb';
         this.qbRootId = 'qb-root';
 		this.qbBannedColumns = [
-			'base__numsample',
+			//'base__numsample',
 			'base__samples',
         ];
         this.allSamples = [];
@@ -611,9 +611,13 @@ class FilterManager {
 		filter = new CravatFilter(filter);
 		outerDiv.append($(getEl('div')).text('Use the query builder to create a set of filter rules'));
 		let qbDiv = makeFilterGroupDiv(filter.variant);
-		qbDiv.children('.filter-element-remove').css('display','none');
+		qbDiv.children('.filter-element-control-div').remove()
+        qbDiv[0].querySelector('.addrule').style.visibility = 'visible'
+        qbDiv[0].querySelector('.addgroup').style.visibility = 'visible'
+        qbDiv[0].querySelector('.passthrough').remove()
 		qbDiv.attr('id', this.qbRootId);
 		outerDiv.append(qbDiv);
+        //addFilterElement(qbDiv, 'rule', undefined)
 	}
 
 	updateVpropUI (filter) {
@@ -845,7 +849,15 @@ function populateFilterSaveNames() {
 				.addClass('filter-list-item-title')
 				.attr('title',filterName)
 				.click(savedFilterClick)
-			)
+            );
+            li.append($(getEl('img'))
+                .attr('src','images/download-material-black.png')
+                .addClass('filter-list-item-export')
+                .attr('title','Export filter to file')
+                .click(()=>{
+                    exportFilter(filterName);
+                })
+            );
 			li.append($(getEl('img'))
 				.attr('src','images/close.png')
 				.addClass('filter-list-item-delete')
@@ -861,10 +873,23 @@ function savedFilterClick(event) {
 	let target = $(this);
 	let filterName = target.text();
 	getSavedFilter(filterName).then((msg) => {
-		filterMgr.updateAll(msg)
+		filterMgr.updateAll(msg['filterSet'])
+        lastUsedFilterName = filterName;
 	});
 }
 
+function exportFilter(filterName) {
+    getSavedFilter(filterName).then((msg)=>{
+        let text = JSON.stringify(msg['filterSet'], null, 2);
+        var link = getEl('a');
+        link.download = filterName+'.json';
+        var blob = new Blob([text], {type: 'text/plain'});
+        link.href = window.URL.createObjectURL(blob);
+        link.click();
+        document.body.removeChild(link);
+    })
+}
+
 function filterDeleteIconClick(event) {
 	let target = $(this);
 	let filterName = target.prop('filterName');
@@ -941,16 +966,16 @@ function makeFilterTab (rightDiv) {
 	let filterCount = $(getEl('img'))
     .attr('src','images/arrow-spinner-static.gif')
     .attr('title','Refresh count')
-	.attr('id','filter-count-btn')
-		.click(function(e) {
+    .attr('id','filter-count-btn')
+        .click(function(e) {
             $(e.target).attr('src','images/arrow-spinner.gif')
-			countFilterVariants();
+            countFilterVariants();
 		}
     );
     loadControls.append(filterCount);
     let countWarning = $(getEl('div'))
         .attr('id','filter-tab-count-warning')
-        .text(`Variants will not load if count is above ${NUMVAR_LIMIT}`)
+        .text(`Variants will not load if count is above ${NUMVAR_LIMIT.toLocaleString()}`)
         .css('display','none');
     loadControls.append(countWarning);
 	let filterApply = $(getEl('button'))
@@ -965,11 +990,12 @@ function makeFilterTab (rightDiv) {
                 if (n <= NUMVAR_LIMIT) {
                     drawingRetrievingDataDiv('filter');
                     loadData(false, null);
+                    loadLayoutSetting(quickSaveName, null);
                 }
             });
 		}
 	);
-	loadControls.append(filterApply);
+    loadControls.append(filterApply);
 	let saveIcon = $(getEl('img'))
 		.attr('id','filter-save')
 		.attr('src','images/save.png')
@@ -980,10 +1006,38 @@ function makeFilterTab (rightDiv) {
 			})
 		});
     loadControls.append(saveIcon);
+    let importInput = $(getEl('input'))
+        .attr('id','filter-import')
+        .attr('type','file')
+        .change(event=>{
+            if (event.target.files.length > 0) {
+                importFilter(event.target.files[0]);
+                event.target.value = null;
+            }
+        })
+        .css('display','none');
+    loadControls.append(importInput);
+    let importLabel = $(getEl('label'))
+        .attr('id','filter-import-label')
+        .attr('for','filter-import')
+        .attr('title','Import filter from file')
+        .append($(getEl('img'))
+            .attr('src','images/upload-material-blue.png')
+        );
+    loadControls.append(importLabel);
     displayFilterCount();
     return true;
 }
 
+function importFilter(file) {
+    var reader = new FileReader();
+    reader.readAsText(file, 'utf-8');
+    reader.addEventListener('load', e => {
+        var filter = JSON.parse(e.target.result);
+        filterMgr.updateAll(filter)
+    });
+}
+
 function countFilterVariants() {
     let countBtn = $('#filter-count-btn')
     gifPath = 'images/arrow-spinner.gif'
@@ -1014,10 +1068,10 @@ function countFilterVariants() {
 }
 
 function displayFilterCount(n) {
-    let t = infomgr.jobinfo['Number of unique input variants'];
+    let t = parseInt(infomgr.jobinfo['Number of unique input variants']);
     n = n===undefined ? t : n;
 	let countDisplay = $('#filter-count-display');
-	countDisplay.text(`${n}/${t} variants`);
+	countDisplay.text(`${n.toLocaleString()}/${t.toLocaleString()} variants`);
     var warnDiv = $('#filter-tab-count-warning');
     if (n > NUMVAR_LIMIT) {
         warnDiv.css('display','');
@@ -1026,6 +1080,23 @@ function displayFilterCount(n) {
     }
 }
 
+function getFilterFile () {
+    makeFilterJson();
+    var fn = 'filter.json';
+    if (lastUsedFilterName != null && lastUsedFilterName != quickSaveName) {
+        fn = lastUsedFilterName + '.json';
+    }
+    var fn = prompt('Filter file name', fn);
+    if (fn == '' || fn == null || fn == undefined) {
+        return;
+    }
+    var a = getEl('a');
+    a.download = fn;
+    var blob = new Blob([JSON.stringify(filterJson, null, 2)], {type: 'text/plain'});
+    a.href = window.URL.createObjectURL(blob);
+    a.click();
+}
+
 function makeFilterJson () {
 	let fjs = {}
 	// Samples
@@ -1556,9 +1627,9 @@ function populateInfoDiv (infoDiv) {
 	var legend = getEl('legend');
 	legend.className = 'section_header';
 	addEl(legend, getTn('Analysis Information\xa0'));
-    var btn = getEl('span');
-    btn.textContent = '\u2a53';
+    var btn = getEl('img');
     btn.style.cursor = 'pointer';
+    btn.src = '/result/images/caret-down.svg'
     btn.addEventListener('click', function (evt) {
         var btn = evt.target;
         var contentDiv = btn.parentElement.nextSibling;
@@ -1566,13 +1637,12 @@ function populateInfoDiv (infoDiv) {
         var btnText = null;
         if (display == 'none') {
             display = 'block';
-            btnText = '\u2a53';
+            btn.src = '/result/images/caret-down.svg'
         } else {
             display = 'none';
-            btnText = '\u2a54';
+            btn.src = '/result/images/caret-right.svg'
         }
         contentDiv.style.display = display;
-        btn.textContent = btnText;
     });
     addEl(legend, btn);
 	addEl(infoDiv, legend);
@@ -1739,10 +1809,10 @@ function onClickWidgetPinButton (evt, tabName) {
     if (pinned) {
         button.classList.remove('pinned');
         button.classList.add('unpinned');
-        button.src = '/result/images/pin.png';
+        //button.src = '/result/images/pin.png';
         $(container).packery('unstamp', widget);
     } else {
-        button.src = '/result/images/pin-2.png';
+        //button.src = '/result/images/pin-2.png';
         button.classList.remove('unpinned');
         button.classList.add('pinned');
         $(container).packery('stamp', widget);
@@ -1754,6 +1824,23 @@ function onClickWidgetCloseButton (tabName, evt) {
     executeWidgetClose(widgetName, tabName, false);
 }
 
+function onClickWidgetDownloadButton (tabName, evt) {
+    var widgetName = evt.target.getAttribute('widgetname');
+    var generator = widgetGenerators[widgetName][tabName]
+    if (generator['exportdata'] != undefined) {
+        var text = generator['exportdata']()
+        var b = new Blob([text], {type: 'text/csv'})
+        var u = window.URL.createObjectURL(b)
+        var a = document.createElement('a')
+        a.setAttribute('href', u)
+        a.setAttribute('download', jobId + '.' + widgetName + '.csv')
+        a.style.display = 'none'
+        document.body.appendChild(a)
+        a.click()
+        document.body.removeChild(a)
+    }
+}
+
 function onClickWidgetCameraButton (tabName, evt) {
     var widgetName = evt.target.getAttribute('widgetname');
     saveWidgetContent(widgetName, tabName);
