@@ -85,7 +85,7 @@ function addSpinnerById(parentDivId, scaleFactor, minDim, spinnerDivId){
 	return spinnerDiv;
 }
 
-function saveFilterSetting (name, useFilterJson) {
+function saveFilterSetting (name, useFilterJson, doPackage) {
 	return new Promise((resolve, reject) => {
 		var saveData = {};
 		if (useFilterJson == undefined) {
@@ -102,6 +102,24 @@ function saveFilterSetting (name, useFilterJson) {
 				writeLogDiv('Filter setting has been saved.');
 				resolve();
                 lastUsedFilterName = name;
+        		if (doPackage != undefined) {
+        			createJobPackage();
+        		}
+			}
+		});
+	})
+}
+
+function createJobPackage() {
+	var packageName = document.getElementById('packageName').value;
+	return new Promise((resolve, reject) => {
+		$.ajax({
+			type: 'GET',
+			async: true,
+			url: '/result/service/jobpackage', 
+			data: {'packagename': packageName, 'job_id': jobId, 'dbpath': dbPath},
+			success: function (response) {
+				writeLogDiv(packageName +'package created');
 			}
 		});
 	})
@@ -383,6 +401,8 @@ function saveLayoutSetting (name, nextAction) {
 			writeLogDiv('Layout setting has been saved.');
             if (nextAction == 'quicksave') {
                 saveFilterSetting(name, true);
+            } else if (nextAction == 'package') {
+                saveFilterSetting(name, true, true);
             }
 		}
     });
@@ -502,7 +522,10 @@ function applyTableSetting (level) {
 						}
 					}
 				}
-                newColModel[i].colModel = newColGroupColModel;
+				//Need to check if columns are there in this job result.  In special case when applying package settings to new data files - some column groups may not be present (like vcf info)
+				if (newColModel[i] != undefined) {
+					newColModel[i].colModel = newColGroupColModel;
+				}
 			}
 		}
 	}
@@ -885,3 +908,57 @@ function checkConnection(failures) {
     ws.onmessage = function (evt) {
     }
 }
+
+function showYesNoDialog(content, yescallback, noSpace, justOk) {
+    var div = document.getElementById('yesnodialog');
+    if (div != undefined) {
+        $(div).remove();
+    }
+    var div = getEl('div');
+    div.id = 'yesnodialog';
+    if (typeof content === 'string') {
+        content = getTn(content);
+    }
+    content.id = 'yesnodialog-contentdiv'
+    addEl(div, content);
+    addEl(div, getEl('br'));
+    var btnDiv = getEl('div');
+    if (justOk) {
+        btnDiv.className = 'buttondiv';
+        var btn = getEl('button');
+        btn.textContent = 'Ok';
+        btn.addEventListener('click', function(evt) {
+            if (yescallback == undefined || yescallback == null) {
+                $('#yesnodialog').remove();
+            } else {
+                $('#yesnodialog').remove();
+                yescallback();
+            }
+        });
+        addEl(btnDiv, btn);
+    } else {
+        btnDiv.className = 'buttondiv';
+        var btn = getEl('button');
+        btn.textContent = 'Yes';
+        btn.addEventListener('click', function(evt) {
+            $('#yesnodialog').remove();
+            yescallback(true);
+        });
+        if (noSpace) {
+            btn.disabled = true;
+            btn.style.backgroundColor = '#e0e0e0';
+        }
+        addEl(btnDiv, btn);
+        var btn = getEl('button');
+        btn.textContent = 'No';
+        btn.addEventListener('click', function(evt) {
+            $('#yesnodialog').remove();
+            yescallback(false);
+        });
+        addEl(btnDiv, btn);
+    }
+    addEl(div, btnDiv);
+    addEl(document.body, div);
+}
+
+
