@@ -33,6 +33,9 @@ function submit () {
         alert('Log in before submitting a job.');
         return;
     }
+    if (systemConf.save_metrics === 'empty') {
+    	informMetrics()
+    }
     formData = new FormData();
     var textInputElem = $('#input-text');
     var textVal = textInputElem.val();
@@ -72,14 +75,25 @@ function submit () {
     }
     var submitOpts = {
         annotators: [],
-        reports: []
+        reports: [],
+    	packages: ''
     }
-    var annotChecks = $('#annotator-select-div').find('input[type=checkbox][kind=module]');
-    for (var i = 0; i<annotChecks.length; i++){
-        var cb = annotChecks[i];
-        if (cb.checked) {
-            submitOpts.annotators.push(cb.value);
-        }
+    var allAnnotsDiv = document.getElementById('annotchoosediv');
+    if (allAnnotsDiv.style.display !== 'none') {
+	    var annotChecks = $('#annotator-select-div').find('input[type=checkbox][kind=module]');
+	    for (var i = 0; i<annotChecks.length; i++){
+	        var cb = annotChecks[i];
+	        if (cb.checked) {
+	            submitOpts.annotators.push(cb.value);
+	        }
+	    }
+    }
+    var allPackagesDiv = document.getElementById('packagechoosediv');
+    if (allPackagesDiv.style.display !== 'none') {
+    	var selPackage = document.getElementById('package-select-dropdown').value;
+    	if (selPackage !== 'none') {
+    		submitOpts.packages = selPackage;
+    	}
     }
     var reportChecks = $('#report-select-div')
                          .find('.checkbox-group-check');
@@ -136,7 +150,7 @@ function submit () {
     } else {
         commitSubmit();
     }
-
+    
     function enableSubmitButton () {
         document.querySelector('#submit-job-button').disabled = false;
     }
@@ -222,7 +236,7 @@ function showUpdateRemoteSpinner () {
 
 function hideUpdateRemoteSpinner () {
     document.querySelector('#update-remote-spinner-div').classList.remove('show');
-    document.querySelector('#update-remote-spinner-div').classList.add('hide');
+    document.querySelector('#update-remote-spinner-div').classList.add('hide'); 
 }
 
 function sortJobs () {
@@ -351,6 +365,23 @@ function getAnnotatorVersionForJob (jobid) {
     return anns;
 }
 
+function informMetrics () {
+    var alertDiv = getEl('div');
+    var span = getEl('span');
+    var a = getEl('a');
+    a.href = 'https://open-cravat.readthedocs.io/en/latest/Metrics.html';
+    a.target = '_blank';
+    a.textContent = 'visit OC Metric documentation';
+    span.textContent = 'OpenCRAVAT gathers metrics to report usage to funders and improve the tool. No private data is collected. For more details, visit our documentation at '
+    addEl(alertDiv, span);
+    addEl(alertDiv, a);
+    showYesNoDialog(alertDiv, null, false, true);
+    var s = document.getElementById('settings_save_metrics');
+    s.checked = true;
+    updateSystemConf(true);
+    systemConf.save_metrics = true;
+}
+
 function onClickJobTableMainTr (evt) {
     if (evt.target.parentElement.classList.contains('job-table-tr') == false) {
         return;
@@ -1070,6 +1101,24 @@ function populateAnnotators () {
     });
 }
 
+function populatePackages () {
+    document.querySelector("#package-select-div").classList.remove("show")
+    return new Promise((resolve, reject) => {
+        $.ajax({
+            url:'/submit/packages',
+            type: 'GET',
+            success: function (data) {
+                document.querySelector("#package-select-div").classList.add("show")
+                GLOBALS.packages = data
+                setTimeout(function () {
+                    buildPackagesSelector();
+                    resolve();
+                }, 100);
+            }
+        })
+    });
+}
+
 function titleCase(str) {
     return str.replace(
       /\w\S*/g,
@@ -1100,6 +1149,7 @@ function buildAnnotatorGroupSelector () {
     tagsCollectedForSubmit.sort((a,b)=>{return tagMembership[b]-tagMembership[a]});
     var annotCheckDiv = document.getElementById('annotator-group-select-div');
     $(annotCheckDiv).empty();
+
     var annotCheckTitleDiv = getEl('div');
     addEl(annotCheckDiv, annotCheckTitleDiv);
     annotCheckTitleDiv.style.display = 'flex';
@@ -1231,6 +1281,71 @@ function buildAnnotatorGroupSelector () {
     });
 }
 
+function buildPackagesSelector () {
+    var packages = GLOBALS.packages;
+    var packageDiv = document.getElementById('package-select-div');
+    if (packages && Object.keys(packages).length===0) {
+        packageDiv.style.display = 'none';
+        document.querySelector('#show-package').style.display = 'none';
+        document.querySelector('#divbanner-annot').style.display = 'none'
+    } else {
+        packageDiv.style.display = '';
+        document.querySelector('#show-package').style.display = '';
+        document.querySelector('#divbanner-annot').style.display = ''
+    }
+    $(packageDiv).empty();
+    var space = getEl('br');
+    
+    var packageDropdown = getEl('select');
+    packageDropdown.id = 'package-select-dropdown';
+	var elnone= getEl('option');
+	elnone.textContent= 'None Selected';
+	elnone.value= 'none';
+	addEl(packageDropdown, elnone);
+
+    for (const [key, value] of Object.entries(packages)) {
+    	var packageKey = `${key}`
+  		var el= getEl('option');
+		el.textContent = value.title;
+		el.value= packageKey;
+		addEl(packageDropdown, el);
+  	}
+    var labelDiv = getEl('div');
+    labelDiv.innerText = 'Available Packages (beta)';
+    labelDiv.style['font-size'] = '1.2em';
+    labelDiv.style['font-weight'] = 'bold';
+    addEl(packageDiv, space);
+    addEl(packageDiv, labelDiv);
+    const pkgHelpLink = getEl('a');
+    pkgHelpLink.href = 'https://open-cravat.readthedocs.io/en/latest/Package.html';
+    pkgHelpLink.target = '_blank';
+    addEl(labelDiv, pkgHelpLink);
+    const pkgHelpImg = getEl('img');
+    pkgHelpImg.src = '/submit/help.png';
+    pkgHelpImg.classList.add('help-img');
+    pkgHelpImg.title = 'Packages are groups of annotators, reports, and filters run together. Click for more information.';
+    addEl(pkgHelpLink, pkgHelpImg);
+    addEl(packageDiv, packageDropdown);
+    addEl(packageDiv, space);
+    document.querySelector('#show-package').addEventListener('click',event=>{
+        packageSelected();
+    })
+}
+
+function packageSelected() {
+    document.getElementById('annotchoosediv').style.display = 'none';
+    document.getElementById('divbanner-annot').style['background-color'] = '#3175b0';
+    document.getElementById('packagechoosediv').style.display = '';
+    document.getElementById('divbanner-pkg').style['background-color'] = 'transparent';
+}
+
+function annotatorSelected() {
+    document.getElementById('annotchoosediv').style.display = '';
+    document.getElementById('divbanner-annot').style['background-color'] = 'transparent';
+    document.getElementById('packagechoosediv').style.display = 'none';
+    document.getElementById('divbanner-pkg').style['background-color'] = '#3175b0';
+}
+
 function buildAnnotatorsSelector () {
     var annotCheckDiv = document.getElementById('annotator-select-div');
     var annotators = GLOBALS.annotators;
@@ -1279,6 +1394,9 @@ function buildAnnotatorsSelector () {
         })
     }
     buildCheckBoxGroup(checkDatas, annotCheckDiv);
+    document.querySelector('#show-annotator').addEventListener('click',event=>{
+        annotatorSelected();
+    })
 }
 
 function buildCheckBoxGroup (checkDatas, parentDiv) {
@@ -1632,6 +1750,12 @@ function openSubmitDiv () {
 function loadSystemConf () {
     $.get('/submit/getsystemconfinfo').done(function (response) {
         systemConf = response.content;
+        var s = document.getElementById('settings_save_metrics');
+        if (systemConf.save_metrics === 'empty') {
+        	s.checked = true;
+        } else {
+        	s.checked = response['content']['save_metrics']
+        } 
         var s = document.getElementById('sysconfpathspan');
         s.value = response['path'];
         var s = document.getElementById('settings_jobs_dir_input');
@@ -1659,7 +1783,7 @@ function onClickSaveSystemConf () {
     updateSystemConf();
 }
 
-function updateSystemConf () {
+function updateSystemConf (setMetrics) {
     $.get('/submit/getsystemconfinfo').done(function (response) {
         var s = document.getElementById('sysconfpathspan');
         response['path'] = s.value;
@@ -1673,20 +1797,28 @@ function updateSystemConf () {
         response['content']['max_num_concurrent_jobs'] = parseInt(s.value);
         var s = document.getElementById('settings_max_num_concurrent_annotators_per_job');
         response['content']['max_num_concurrent_annotators_per_job'] = parseInt(s.value);
+        var s = document.getElementById('settings_save_metrics');
+        var optout = false;
+        if ((response['content']['save_metrics'] !== false) && (s.checked === false)) {
+            optout = true;
+        }
+        response['content']['save_metrics'] = s.checked;
         $.ajax({
             url:'/submit/updatesystemconf',
-            data: {'sysconf': JSON.stringify(response['content'])},
+            data: {'sysconf': JSON.stringify(response['content']), 'optout': optout},
             type: 'GET',
             success: function (response) {
                 if (response['success'] == true) {
-                    var mdiv = getEl('div');
-                    var span = getEl('span');
-                    span.textContent = 'System configuration has been updated.';
-                    addEl(mdiv, span);
-                    addEl(mdiv, getEl('br'));
-                    addEl(mdiv, getEl('br'));
-                    var justOk = true;
-                    showYesNoDialog(mdiv, null, false, justOk);
+                	if (typeof setMetrics === 'undefined') {
+	                    var mdiv = getEl('div');
+	                    var span = getEl('span');
+	                    span.textContent = 'System configuration has been updated.';
+	                    addEl(mdiv, span);
+	                    addEl(mdiv, getEl('br'));
+	                    addEl(mdiv, getEl('br'));
+	                    var justOk = true;
+	                    showYesNoDialog(mdiv, null, false, justOk);
+                	}
                 } else {
                     var mdiv = getEl('div');
                     var span = getEl('span');
@@ -1835,14 +1967,6 @@ function populateMultInputsMessage() {
         var $fileListDiv = $('#mult-inputs-list');
         for (var i=0; i<files.length; i++) {
             var file = files[i];
-            if (file.name.indexOf(' ') > -1) {
-                var alertDiv = getEl('div');
-                var span = getEl('span');
-                span.textContent = 'Space in file names is not allowed.';
-                addEl(alertDiv, span);
-                showYesNoDialog(alertDiv, null, false, true);
-                return;
-            }
             if (inputFileList.indexOf(file.name) == -1) {
                 var sdiv = getEl('div');
                 var span = getEl('span');
@@ -2107,8 +2231,8 @@ function populateCaseControl (outer) {
     let helpIcon = $(getEl('a'))
         .attr('id','case-control-help')
         .attr('target','_blank')
-        .attr('href','https://github.com/KarchinLab/open-cravat/wiki/Case-Control')
-        .append($(getEl('img')).attr('src','../help.png').attr('id','case-control-help-img'));
+        .attr('href','https://open-cravat.readthedocs.io/en/latest/Case-Control.html')
+        .append($(getEl('img')).attr('src','../help.png').attr('class','help-img'));
     d1.append(helpIcon);
     let d2 = $(getEl('div'))
     wrapper.append(d2);
