@@ -7,6 +7,7 @@ import json
 from cravat import admin_util as au
 from cravat.config_loader import ConfigLoader
 from cravat.cravat_class import run_cravat_job
+from cravat import cravat_metrics as metrics
 import sys
 import traceback
 import shutil
@@ -293,12 +294,12 @@ async def resubmit (request):
     run_args.append('-l')
     run_args.append(assembly)
     # Reports
-    run_args.extend(['--skip', 'reporter'])
+    #run_args.extend(['--skip', 'reporter'])
     # Note
     if note != '':
         run_args.append('--note')
         run_args.append(note)
-    run_args.append('--temp-files')
+    run_args.append('--keep-status')
     if cc_cohorts_path != '':
         run_args.extend(['--module-option',f'casecontrol.cohorts={cc_cohorts_path}'])
     global job_queue
@@ -391,16 +392,22 @@ async def submit (request):
     else:
         for fn in input_fnames:
             run_args.append(os.path.join(job_dir, fn))
-    # Annotators
-    if 'annotators' in job_options and len(job_options['annotators']) > 0 and job_options['annotators'][0] != '':
-        annotators = job_options['annotators']
-        annotators.sort()
-        run_args.append('-a')
-        run_args.extend(annotators)
+            
+    ''' Packages or Annotators '''
+    if 'packages' in job_options and job_options['packages'] != '':
+        packs = job_options['packages'] 
+        run_args.append('--package')
+        run_args.append(packs)
     else:
-        annotators = ''
-        run_args.append('-e')
-        run_args.append('all')
+        if 'annotators' in job_options and len(job_options['annotators']) > 0 and job_options['annotators'][0] != '':
+            annotators = job_options['annotators']
+            annotators.sort()
+            run_args.append('-a')
+            run_args.extend(annotators)
+        else:
+            annotators = ''
+            run_args.append('-e')
+            run_args.append('all')
     # Liftover assembly
     run_args.append('-l')
     if 'assembly' in job_options:
@@ -416,8 +423,8 @@ async def submit (request):
     if 'reports' in job_options and len(job_options['reports']) > 0:
         run_args.append('-t')
         run_args.extend(job_options['reports'])
-    else:
-        run_args.extend(['--skip', 'reporter'])
+    #else:
+    #    run_args.extend(['--skip', 'reporter'])
     # Note
     if 'note' in job_options:
         note = job_options['note']
@@ -431,7 +438,7 @@ async def submit (request):
     if servermode:
         run_args.append('--writeadmindb')
         run_args.extend(['--jobid', job_id])
-    run_args.append('--temp-files')
+    run_args.append('--keep-status')
     if cc_cohorts_path is not None:
         run_args.extend(['--module-option',f'casecontrol.cohorts={cc_cohorts_path}'])
     global job_queue
@@ -461,7 +468,7 @@ async def submit (request):
     status_json['reports'] = []
     pkg_ver = au.get_current_package_version()
     status_json['open_cravat_version'] = pkg_ver
-    status_json['annotators'] = annotators
+#    status_json['annotators'] = annotators
     if cc_cohorts_path is not None:
         status_json['cc_cohorts_path'] = cc_cohorts_path
     else:
@@ -497,6 +504,21 @@ def get_annotators(request):
                                 }
     return web.json_response(out)
 
+def get_packages(request):
+    out = {}
+    for local_info in au.get_local_module_infos(types=['package']):
+        package_name = local_info.name
+        if local_info.type == 'package':
+            out[package_name] = {
+                                'name':package_name,
+                                'version':local_info.version,
+                                'type':local_info.type,
+                                'title':local_info.title,
+                                'description':local_info.description,
+                                'developer': local_info.developer
+                                }
+    return web.json_response(out)
+
 def find_files_by_ending (d, ending):
     fns = os.listdir(d)
     files = []
@@ -697,7 +719,7 @@ async def generate_report(request):
     job_id = request.match_info['job_id']
     report_type = request.match_info['report_type']
     job_db_path = await filerouter.job_db(request, job_id)
-    run_args = ['oc', 'report', job_db_path]
+    run_args = ['report', job_db_path]
     run_args.extend(['-t', report_type])
     if job_id not in report_generation_ps:
         report_generation_ps[job_id] = {}
@@ -706,7 +728,7 @@ async def generate_report(request):
     wf = open(tmp_flag_path, 'w')
     wf.write(report_type)
     wf.close()
-    p = await asyncio.create_subprocess_shell(' '.join(run_args), stderr=asyncio.subprocess.PIPE)
+    p = await asyncio.create_subprocess_exec('oc', *run_args, stderr=asyncio.subprocess.PIPE)
     out, err = await p.communicate()
     os.remove(tmp_flag_path)
     if report_type in report_generation_ps[job_id]:
@@ -762,6 +784,10 @@ async def update_system_conf (request):
             return web.json_response({'success': False, 'mgs': 'Only logged-in admin can change the settings.'})
     queries = request.rel_url.query
     sysconf = json.loads(queries['sysconf'])
+    optout = queries['optout']
+    if optout == 'true':
+        metricObj = metrics.cravatMetrics()
+        metricObj.do_opt_out()
     try:
         success = au.update_system_conf_file(sysconf)
         if 'modules_dir' in sysconf:
@@ -976,21 +1002,26 @@ def fetch_job_queue (job_queue, run_jobs_info):
 
     async def job_worker_main ():
         while True:
-            job_tracker.clean_jobs(None)
-            job_tracker.run_available_jobs()
             try:
-                qitem = job_queue.get_nowait()
-                cmd = qitem['cmd']
-                if cmd == 'submit':
-                    job_tracker.add_job(qitem)
-                elif cmd == 'delete':
-                    await job_tracker.delete_job(qitem)
-                elif cmd == 'set_max_num_concurrent_jobs':
-                    job_tracker.set_max_num_concurrent_jobs(qitem)
-            except Empty:
-                pass
-            finally:
-                await asyncio.sleep(1)
+                job_tracker.clean_jobs(None)
+                job_tracker.run_available_jobs()
+                try:
+                    qitem = job_queue.get_nowait()
+                    cmd = qitem['cmd']
+                    if cmd == 'submit':
+                        job_tracker.add_job(qitem)
+                    elif cmd == 'delete':
+                        await job_tracker.delete_job(qitem)
+                    elif cmd == 'set_max_num_concurrent_jobs':
+                        job_tracker.set_max_num_concurrent_jobs(qitem)
+                except Empty:
+                    pass
+                finally:
+                    await asyncio.sleep(1)
+            except:
+                import traceback
+                print('Exception in job fetch loop')
+                traceback.print_exc()
 
     main_loop = asyncio.new_event_loop()
     job_tracker = JobTracker(main_loop)
@@ -1183,8 +1214,8 @@ async def update_result_db (request):
     job_dir = await filerouter.job_dir(request, job_id)
     fns = find_files_by_ending(job_dir, '.sqlite')
     db_path = os.path.join(job_dir, fns[0])
-    cmd = ['oc', 'util', 'update-result', db_path]
-    p = await asyncio.create_subprocess_shell(' '.join(cmd))
+    cmd = ['util', 'update-result', db_path]
+    p = await asyncio.create_subprocess_exec('oc', *cmd)
     await p.wait()
     compatible_version, db_version, oc_version = cravat.util.is_compatible_version(db_path)
     if compatible_version:
@@ -1233,6 +1264,7 @@ interval_log_single_api_access = 60
 routes = []
 routes.append(['POST','/submit/submit',submit])
 routes.append(['GET','/submit/annotators',get_annotators])
+routes.append(['GET','/submit/packages',get_packages])
 routes.append(['GET','/submit/jobs',get_all_jobs])
 routes.append(['GET','/submit/jobs/{job_id}',view_job])
 routes.append(['DELETE','/submit/jobs/{job_id}',delete_job])
